#
# Copyright (c) 2022 PrajjuS <theprajjus@gmail.com>.
#
# This file is part of NoobStuffs
# (see http://github.com/PrajjuS/NoobStuffs).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#

import string

import requests

MAGISK_DICT = {
    "Stable": "https://raw.githubusercontent.com/topjohnwu/magisk-files/master/stable.json",
    "Beta": "https://raw.githubusercontent.com/topjohnwu/magisk-files/master/beta.json",
    "Canary": "https://raw.githubusercontent.com/topjohnwu/magisk-files/master/canary.json",
}


def get_magisk(version_type: str):
    if string.capwords(version_type) in list(MAGISK_DICT.keys()):
        data = requests.get(url=MAGISK_DICT[string.capwords(version_type)]).json()
        return {
            "type": string.capwords(version_type),
            "version": data["magisk"]["version"],
            "download": data["magisk"]["link"],
            "changelog": data["magisk"]["note"],
        }
    else:
        raise Exception("Oh noo you gave wrong version type.")


def get_all_magisks():
    releases = dict()
    for name, release_url in MAGISK_DICT.items():
        data = requests.get(release_url).json()
        releases.update(
            {
                name: {
                    "version": data["magisk"]["version"],
                    "download": data["magisk"]["link"],
                    "changelog": data["magisk"]["note"],
                },
            },
        )
    return releases


print(get_magisk("sTable"))



#
# Copyright (c) 2022 PrajjuS <theprajjus@gmail.com>.
#
# This file is part of NoobStuffs
# (see http://github.com/PrajjuS/NoobStuffs).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#

"""Logging Library"""

import logging

STREAM_FORMAT = "[%(levelname)s] - %(name)s: %(message)s"
LOG_FORMAT = "[%(levelname)s] [%(asctime)s] - %(name)s: %(message)s"
VERBOSE_FORMAT = "[%(levelname)s] [%(asctime)s] [%(filename)s %(lineno)d] %(funcName)s - %(name)s: %(message)s"


def setup_logging(name: str, verbose: bool = False):
    if verbose:
        stream_formatter = logging.Formatter(STREAM_FORMAT)
        file_formatter = logging.Formatter(LOG_FORMAT)
        verbose_formatter = logging.Formatter(VERBOSE_FORMAT)
        stream_handler = logging.StreamHandler()
        file_handler = logging.FileHandler(f"{name}.log")
        verbose_handler = logging.FileHandler(f"{name}-DEBUG.log")
        stream_handler.setFormatter(stream_formatter)
        file_handler.setFormatter(file_formatter)
        verbose_handler.setFormatter(verbose_formatter)
        stream_handler.setLevel(logging.INFO)
        file_handler.setLevel(logging.INFO)
        verbose_handler.setLevel(logging.DEBUG)
        logging.basicConfig(
            handlers=[stream_handler, file_handler, verbose_handler],
            level=logging.DEBUG,
        )
        logger = logging.getLogger(name=name)
    else:
        stream_formatter = logging.Formatter(STREAM_FORMAT)
        file_formatter = logging.Formatter(LOG_FORMAT)
        stream_handler = logging.StreamHandler()
        file_handler = logging.FileHandler(f"{name}.log")
        stream_handler.setFormatter(stream_formatter)
        file_handler.setFormatter(file_formatter)
        stream_handler.setLevel(logging.INFO)
        file_handler.setLevel(logging.INFO)
        logging.basicConfig(handlers=[stream_handler, file_handler], level=logging.INFO)
        logger = logging.getLogger(name=name)
    return logger



#
# Copyright (c) 2022 PrajjuS <theprajjus@gmail.com>.
#
# This file is part of NoobStuffs
# (see http://github.com/PrajjuS/NoobStuffs).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#

from logging import getLogger
from random import choices
from string import ascii_letters
from time import sleep

from telegraph import Telegraph
from telegraph.exceptions import RetryAfterError

LOGGER = getLogger("TelegraphHelper")


class TelegraphHelper:
    def __init__(self, author_name: str, author_url: str):
        self.telegraph = Telegraph()
        self.short_name = "".join(choices(population=ascii_letters, k=5))
        self.author_name = author_name
        self.author_url = author_url
        self.create_account()

    def create_account(self):
        LOGGER.info(f"Creating account: {self.author_name}")
        self.telegraph.create_account(
            short_name=self.short_name,
            author_name=self.author_name,
            author_url=self.author_url,
        )

    def create_page(self, title: str, content: str):
        LOGGER.info(f"Creating page: {title}")
        try:
            return self.telegraph.create_page(
                title=title,
                html_content=content,
                author_name=self.author_name,
                author_url=self.author_url,
            )
        except RetryAfterError as err:
            LOGGER.error(
                f"Telegraph Flood control exceeded, sleeping for {err.retry_after} seconds.",
            )
            sleep(err.retry_after)
            return self.create_page(title, content)

    def upload_file(self, path: str):
        LOGGER.info(f"Uploading file: {path}")
        try:
            return self.telegraph.upload_file(path)
        except RetryAfterError as err:
            LOGGER.error(
                f"Telegraph Flood control exceeded, sleeping for {err.retry_after} seconds.",
            )
            sleep(err.retry_after)
            return self.upload_file(path)



#
# Copyright (c) 2022 PrajjuS <theprajjus@gmail.com>.
#
# This file is part of NoobStuffs
# (see http://github.com/PrajjuS/NoobStuffs).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#

"""Text Formatter Library"""

from typing import Optional

from .escape import TextEscape


class HTML:
    def bold(text: str):
        return f"<b>{TextEscape.html_escape(text)}</b>"

    def mono(text: str):
        return f"<code>{TextEscape.html_escape(text)}</code>"

    def pre(text: str):
        return f"<pre>{TextEscape.html_escape(text)}</pre>"

    def italic(text: str):
        return f"<i>{TextEscape.html_escape(text)}</i>"

    def underline(text: str):
        return f"<u>{TextEscape.html_escape(text)}</u>"

    def strike(text: str):
        return f"<s>{TextEscape.html_escape(text)}</s>"

    def spoiler(text: str):
        return f"<spoiler>{TextEscape.html_escape(text)}</spoiler>"

    def heading(text: str, size: int):
        if size == 1:
            return f"<h1>{TextEscape.html_escape(text)}</h1>"

        elif size == 2:
            return f"<h2>{TextEscape.html_escape(text)}</h2>"

        elif size == 3:
            return f"<h3>{TextEscape.html_escape(text)}</h3>"

        elif size == 4:
            return f"<h4>{TextEscape.html_escape(text)}</h4>"

        elif size == 5:
            return f"<h5>{TextEscape.html_escape(text)}</h5>"

        elif size == 6:
            return f"<h6>{TextEscape.html_escape(text)}</h6>"

        else:
            raise ValueError("Invalid size, use sizes between 1 to 6")

    def hyperlink(text: str, link: str):
        return f"<a href='{TextEscape.html_escape(link)}'>{TextEscape.html_escape(text)}</a>"

    def mention(text: str, uid: int):
        return f"<a href='tg://user?id={uid}'>{TextEscape.html_escape(text)}</a>"

    def invisible_link(link: str):
        return f"<a href='{TextEscape.html_escape(link)}'>\u2063</a>"

    def colon_item(key: str, value: str, mono_value: Optional[bool] = True):
        text = f"<b>{key}:</b> "
        text += f"<code>{value}</code>" if mono_value else f"{value}"
        return text


class MARKDOWN:
    def bold(text: str):
        return f"*{TextEscape.markdown_escape(text)}*"

    def mono(text: str):
        return f"`{TextEscape.markdown_escape(text)}`"

    def pre(text: str):
        return f"```{TextEscape.markdown_escape(text)}```"

    def italic(text: str):
        return f"_{TextEscape.markdown_escape(text)}_"

    def underline(text: str):
        return f"__{TextEscape.markdown_escape(text)}__"

    def strike(text: str):
        return f"~{TextEscape.html_escape(text)}~"

    def spoiler(text: str):
        return f"||{TextEscape.markdown_escape(text)}||"

    def heading(text: str, size: int):
        if size == 1:
            return f"#{TextEscape.html_escape(text)}#"

        elif size == 2:
            return f"##{TextEscape.html_escape(text)}##"

        elif size == 3:
            return f"###{TextEscape.html_escape(text)}###"

        elif size == 4:
            return f"####{TextEscape.html_escape(text)}####"

        elif size == 5:
            return f"#####{TextEscape.html_escape(text)}#####"

        elif size == 6:
            return f"######{TextEscape.html_escape(text)}######"

        else:
            raise ValueError("Invalid size, use sizes between 1 to 6")

    def hyperlink(text: str, link: str):
        return (
            f"[{TextEscape.markdown_escape(text)}]({TextEscape.markdown_escape(link)})"
        )

    def mention(text: str, uid: int):
        return f"[{TextEscape.markdown_escape(text)}](tg://user?id={uid})"

    def invisible_link(link: str):
        return f"[\u2063]({TextEscape.html_escape(link)})"

    def colon_item(key: str, value: str, mono_value: Optional[bool] = True):
        text = f"*{key}:* "
        text += f"`{value}`" if mono_value else f"{value}"
        return text



#
# Copyright (c) 2022 PrajjuS <theprajjus@gmail.com>.
#
# This file is part of NoobStuffs
# (see http://github.com/PrajjuS/NoobStuffs).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#

"""Date Time Library"""

import datetime
from typing import Optional

import pytz

TIME_FORMAT = "%H:%M:%S"
DATE_FORMAT = "%Y-%m-%d"
DAY_FORMAT = "%A"


def dtnow(tzinfo: str = "Asia/Kolkata"):
    dt = datetime.datetime.now(pytz.timezone(tzinfo))
    TIME = dt.strftime(TIME_FORMAT)
    DATE = dt.strftime(DATE_FORMAT)
    DAY = dt.strftime(DAY_FORMAT)
    return {
        "time": TIME,
        "date": DATE,
        "day": DAY,
    }


def fromtimestamp(timestamp: int, tzinfo: Optional[str] = None):
    if tzinfo == None:
        dt = datetime.datetime.fromtimestamp(timestamp)
    else:
        dt = datetime.datetime.fromtimestamp(timestamp, pytz.timezone(tzinfo))
    TIME = dt.strftime(TIME_FORMAT)
    DATE = dt.strftime(DATE_FORMAT)
    DAY = dt.strftime(DAY_FORMAT)
    return {
        "time": TIME,
        "date": DATE,
        "day": DAY,
    }


def fromdatetime(date_time: str, dtformat: str):
    dt = datetime.datetime.strptime(date_time, dtformat)
    return dt.timestamp()



#
# Copyright (c) 2022 PrajjuS <theprajjus@gmail.com>.
#
# This file is part of NoobStuffs
# (see http://github.com/PrajjuS/NoobStuffs).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#


class TextEscape:
    def markdown_escape(text: str):
        text = text.replace("*", "\\*")
        text = text.replace("`", "\\`")
        text = text.replace("_", "\\_")
        text = text.replace("~", "\\~")
        text = text.replace(">", "\\>")
        text = text.replace("[", "\\[")
        text = text.replace("]", "\\]")
        text = text.replace("(", "\\(")
        text = text.replace(")", "\\)")
        return text

    def html_escape(text: str):
        text = text.replace("&", "&amp;")
        text = text.replace("<", "&lt;")
        text = text.replace(">", "&gt;")
        text = text.replace('"', "&quot;")
        text = text.replace("'", "&#x27;")
        return text



#
# Copyright (c) 2022 PrajjuS <theprajjus@gmail.com>.
#
# This file is part of NoobStuffs
# (see http://github.com/PrajjuS/NoobStuffs).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#

from logging import getLogger

from github import Github, GithubException, InputGitAuthor, UnknownObjectException

LOGGER = getLogger("GithubHelper")


class GithubHelper:
    def __init__(self, gh_token: str, username: str, email: str):
        self.g = Github(login_or_token=gh_token)
        self.user = self.g.get_user()
        self.committer = InputGitAuthor(name=username, email=email)

    def cleanup_repo(self, repo: str):
        LOGGER.info(f"Cleaning up repo: {repo}")
        try:
            r = self.g.get_repo(full_name_or_id=repo)
            r.delete()
        except UnknownObjectException as e:
            LOGGER.error(f"Error while cleaning up repo {repo}: {e.data}")

    def fork_repo(self, repo: str):
        LOGGER.info(f"Forking repo: {repo}")
        try:
            r = self.g.get_repo(full_name_or_id=repo)
            return self.user.create_fork(repo=r)
        except GithubException as e:
            LOGGER.error(f"Error while forking repo {repo}: {e.data}")

    def empty_commit(self, repo: str, path: str, commit_message: str, branch: str):
        LOGGER.info(
            f"Committing changes in repo: repo={repo}, message={commit_message}, branch={branch}",
        )
        try:
            r = self.g.get_repo(full_name_or_id=repo)
            contents = r.get_contents(path)
            return r.update_file(
                path=contents.path,
                message=commit_message,
                content=contents.decoded_content.decode(),
                sha=contents.sha,
                branch=branch,
                committer=self.committer,
            )
        except GithubException as e:
            LOGGER.error(f"Error while committing changes in repo {repo}: {e.data}")

    def commit_changes(
        self,
        repo: str,
        path: str,
        commit_message: str,
        content: str,
        branch: str,
    ):
        LOGGER.info(
            f"Committing changes in repo: repo={repo}, file={path} content={content} message={commit_message}, branch={branch}",
        )
        try:
            r = self.g.get_repo(full_name_or_id=repo)
            contents = r.get_contents(path)
            return r.update_file(
                path=contents.path,
                message=commit_message,
                content=content,
                sha=contents.sha,
                branch=branch,
                committer=self.committer,
            )
        except GithubException as e:
            LOGGER.error(f"Error while committing changes in repo {repo}: {e.data}")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['NoobStuffs',
 'NoobStuffs.libandroid',
 'NoobStuffs.libdatetime',
 'NoobStuffs.libformatter',
 'NoobStuffs.libgithub',
 'NoobStuffs.liblogging',
 'NoobStuffs.libpasty',
 'NoobStuffs.libtelegraph']

package_data = \
{'': ['*']}

install_requires = \
['PyGithub>=1.55,<2.0', 'pytz>=2022.1,<2023.0', 'requests>=2.28.0,<3.0.0']

setup_kwargs = {
    'name': 'noobstuffs',
    'version': '1.0.7',
    'description': 'Python Library for PrajjuS Projects',
    'long_description': '# NoobStuffs\n\n[![PyPi version](https://img.shields.io/pypi/v/noobstuffs)](https://pypi.org/project/noobstuffs/)\n[![Codacy Badge](https://app.codacy.com/project/badge/Grade/e73fc5d0a6b64bb8a6c286f88c1dc590)](https://www.codacy.com/gh/PrajjuS/NoobStuffs/dashboard?utm_source=github.com&amp;utm_medium=referral&amp;utm_content=PrajjuS/NoobStuffs&amp;utm_campaign=Badge_Grade)\n\n>Python3 library which consists of codes that are frequently used for my personal projects\n\n## Installation\n\n```\npip install noobstuffs\n```\n\n## Credits\n\nInspired by [sebaubuntu_libs](https://github.com/SebaUbuntu/sebaubuntu_libs)\n',
    'author': 'PrajjuS',
    'author_email': 'theprajjus@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/PrajjuS/NoobStuffs',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



#
# Copyright (c) 2022 PrajjuS <theprajjus@gmail.com>.
#
# This file is part of NoobStuffs
# (see http://github.com/PrajjuS/NoobStuffs).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#

"""Pasty Library"""

import requests

PASTE_URL = "https://pasty.lus.pm"


def paste_content(content: str):
    data = {
        "content": content,
    }
    re = requests.post(url=f"{PASTE_URL}/api/v2/pastes", json=data)
    data = re.json()
    return {
        "url": f"{PASTE_URL}/{data['id']}",
        "raw": f"{PASTE_URL}/{data['id']}/raw",
        "modificationToken": data["modificationToken"],
    }


def get_content(paste_id: str):
    re = requests.get(url=f"{PASTE_URL}/api/v2/pastes/{paste_id}")
    data = re.json()
    return {
        "url": f"{PASTE_URL}/{data['id']}",
        "content": data["content"],
        "created": data["created"],
    }
